*     JLdL 24Nov14.
*     
*     Add up the center series for the Fourier conjugate function of
*     _ the first very slow-converging wave until one gets a given
*     _ precision level, and count the terms.
*     
*     Note: this program is meant to be called from within the
*     _ shell script test_vs1w_conj_cent.test.
*     
*     Parameters:
*     _ mpnts: a fixed parameter for the variable npnts.
*     
*     Variables:
*     _ exfun: the (almost) exact value of the function;
*     _ ermax: the maximum error acceptable in the result (input);
*     _ nspec: the number of special points showing in the graph;
*     _ onepi: the value of \pi;
*     _ snorm: the overall normalization constant of the series;
*     _ mktrm: the maximum number of terms to add up on the series;
*     _ npnts: the number of points in the x half interval;
*     _ delta: the increment of x within its interval;
*     _ trmsn: the graph-wise total number of series terms added;
*     _ mtrms: the single largest number of series terms added;
*     _ ixvar: the integer x variable (loop counter);
*     _ rxvar: the current real value of x;
*     _ xnorm: the x-dependent normalization constant;
*     _ exact: the exact value of the function at x;
*     _ acser: the accumulator for the series of the function;
*     _ iexit: the series loop exit flag;
*     _ ijtrm: the counter for the series terms (loop counter);
*     _ iktrm: the value of k for the series terms;
*     _ coef:  temporary for the coefficients of the series;
*     _ term:  temporary for a term of the series;
*     _ fnvar: the value of the function;
*     _ error: the current value of the error;
*     _ jtrms: the number of series terms added at the current point.
*     
      PROGRAM TEST_VS1W_CONJ_CENT
*     
*     Include global integer parameters;
*     _ this file defines mpnts.
      include 'global_params.f'
*     
*     Double precision everything.
      implicit double precision(a-h,o-z)
*     
*     Storage array for the exact function.
      dimension exfun(-mpnts:mpnts)
*     
*     Set the maximum acceptable error; read from stdin in filter mode.
      read(*,*) ermax
*     
*     Define the number of special points in graph; in this case there
*     _ are 1 special point(s), showing as 1 point(s) in the graph.
      nspec=1
*     
*     Define constants.
      onepi=dacos(-1.0d0)
*     
*     Define the global normalization for the series.
      snorm=1.0d0/onepi
*     
*     Set the maximum number of series terms to add up.
      mktrm=10000000
*     
*     Set the number of data points.
      npnts=mpnts
*     
*     Define the point spacing along x, and the range of x.
      delta=onepi/dble(npnts)
*     
*     Initialize the accumulator for the total numbers of terms.
      trmsn=0.0d0
*     
*     Initialize the storage for the maximum numbers of terms.
      mtrms=0
*     
*     Initialize the storage array.
      do ixvar=-npnts,npnts
         exfun(ixvar)=0.0d0
      end do
*     
*     Open the file containing the exact function.
      open(unit=1,file='exact.dat',status="old",iostat=ioerr)
      if (ioerr.ne.0) then
         if (ioerr.eq.2) then
            write(*,*) 'ERROR: file exact.dat not found'
         else
            write(*,*) 'ERROR: unknown error number ',ioerr
         end if
         stop
      end if
*     
*     Read in the exact function.
      itemp=0
      do while(.true.)
         itemp=temp+1
         read(1,*,err=100,end=100) ixvar,fexct
         exfun(ixvar)=fexct
      end do
 100  close(unit=1)
*     
*     Open an output file for the graph, for monitoring purposes.
      open(unit=1,file='graph.dat')
*     
*     Loop over the x interval.
      do ixvar=-npnts,npnts
*     
*     Avoid the special points.
         if ( ixvar.ne.0 ) then
*     
*     Define the current value of x.
            rxvar=dble(ixvar)*delta
*     
*     Define the x-dependent normalization.
            xnorm=snorm/dsin(0.5d0*rxvar)
*     
*     Define the exact value of the function.
            exact=exfun(ixvar)
*     
*     Initialize the series accumulator for the function.
            acser=0.0d0
*     
*     Initialize the exit flag.
            iexit=0
*     
*     Initialize a loop counter for the terms.
            ijtrm=1
            iktrm=ijtrm
*     
*     Add up the series; loop until the value of the error decays below
*     _ the precision threshold; this has to remain so for 10 terms, in
*     _ order to filter out accidental hits; also monitor the loop
*     _ counter as an extreme fall-back measure.
            do while
     *           (
     *           iexit.lt.10
     *           .and.
     *           ijtrm.lt.mktrm
     *           )
*     
*     Calculate the series.
               coef=dsqrt(dble(iktrm))*(dble(iktrm)+1.0d0)
     *              +dble(iktrm)*dsqrt(dble(iktrm)+1.0d0)
               coef=1.0d0/coef
               term=coef*dsin((dble(iktrm)+0.5d0)*rxvar)
               acser=acser+term
*     
*     Calculate the function.
               fnvar=xnorm*(-dsin(0.5d0*rxvar)+acser)
*     
*     Calculate the error.
               error=dabs(exact-fnvar)
*     
*     Check the error and update the exit flag.
               if (error.lt.ermax) then
                  iexit=iexit+1
               else
                  iexit=0
               end if
*     
*     Increment the counter.
               ijtrm=ijtrm+1
               iktrm=ijtrm
*     
            end do
*     
*     We should not have the maximum number of term be actually reached.
            if (ijtrm.ge.mktrm) then
               write(*,*) 'ERROR: maximum term count reached: ',ijtrm
               write(*,*) 'ERROR: aborting run; must update the program'
               stop
            end if
*     
*     Record the effective number of terms.
            jtrms=ijtrm-1-10
*     
*     Accumulate the total number of terms.
            trmsn=trmsn+dble(jtrms)
*     
*     Record the maximum number of terms.
            if (jtrms.ge.mtrms) then
               mtrms=jtrms
            end if
*     
*     Write the monitoring graphics data to the output file.
            write(1,*) rxvar,fnvar
*     
         end if
*     
      end do
*     
*     Write to stdout.
      write(*,*) 'maximum error: ',ermax
      write(*,*) 'average terms: ',trmsn/dble(2*npnts+1-nspec)
      write(*,*) 'maximum terms: ',dble(mtrms)
*     
      END
